; ********************************************* ;
pro scattering_function,x,p,yfit,_Extra = extra
bg = p[0] & area_el = p[1] & cen_el = p[2]
area_iel = p[3] & cen_iel = p[4] & fwhm_iel = p[5]
y_iel = lorentzian(x,area = area_iel,cen = cen_iel,fwhm = fwhm_iel)
nlo = 3 & nhi = n_elements(x) - nlo - 1
dx = x[1] - x[0]
con = convol(*extra.res_ptr,reverse(y_iel[nlo:nhi]),/edge_truncate) * dx
yfit = bg+gaussian(x,area = area_el,cen = cen_el,fwhm = 1.0)+con
end
; ********************************************* ;
pro analyze_fake_data
; First read in the data
filename = sourceroot()+'fake_data.txt'
count = 0L
dum1 = (dum2 = (dum3 = 0.0))
openr,lun,filename,/get_lun
while not eof(lun) do begin
   readf,lun,dum1,dum2,dum3
   if count eq 0L then begin
      x = dum1 & y = dum2 & dy = dum3
   endif else begin
      x = [x,dum1] & y = [y,dum2] & dy = [dy,dum3]
   endelse
   count++
endwhile
free_lun,lun,/force

plot,x,y,psym = 4
errplot,x,y-dy,y+dy,width = 0.0

; We will assume that we know that the resolution
; function is a Gaussian with a FWHM of 1.0 and is
; defined over the same interval as the data.
res = gaussian(x,area = 1.0,center = 0.0, fwhm = 1.0)
; Create a pointer to the resolution function because we
; will not want to pass around the vector and keep making
; copies of it.  A single reference to a heap variable
; is all that's necessary.
res_ptr = ptr_new(res)

; Pick some initial guesses for the fit
bg = 10.0 & area_el = 500.0 & cen_el = 0.0
area_iel = 100.0 & cen_iel = 5.0 & fwhm_iel = 1.0
p = [bg,area_el,cen_el,area_iel,cen_iel,fwhm_iel]
; Impose constraints on each of the positive-definite
; parameters that they be greater than zero.
parinfo = replicate({limited:[0,0],limits:[0D,0D]},n_elements(p))
parinfo[0].limited[0] = 1
parinfo[0].limits[0] = 0.D
parinfo[1].limited[0] = 1
parinfo[1].limits[0] = 0.D
parinfo[3].limited[0] = 1
parinfo[3].limits[0] = 0.D
parinfo[5].limited[0] = 1
parinfo[5].limits[0] = 0.D

functargs = {res_ptr:res_ptr}
yfit = mpcurvefit(x,y,1d/dy^2,p, sigma, parinfo = parinfo, $
   function_name = 'scattering_function', $
   /noderivative,functargs = functargs    )

oplot,x,yfit,psym = 0,thick = 2.0
for i = 0,n_elements(p)-1 do $
   print,strtrim(string(p[i]),2)+' +/- '+strtrim(string(sigma[i]),2)
ptr_free,res_ptr
end