; fourierProgram.pro
;
; Demonstration of the Fourier series approximation to
; a rectangle function as the number of terms in the
; series expansion increases.
;
; *********************************
pro fourierProgram
; Use the DEVICE procedure with the GET_SCREEN_SIZE
; keyword to determine the size (in pixels) of the
; screen.  Then make the window size 1/3 of this in
; both dimensions.
device,get_screen_size = screen_size
winxsize = 400 & winysize = 400
xsize = fix(screen_size[0]/3.0)
ysize = fix(screen_size[1]/3.0)

nx = 500
x = makepts(xlo = -0.5, xhi = 1.5*!pi, npts = nx)
y = heaviside(x)-heaviside(x-!pi) ; the function

nterms = 100 ; # of terms in the FS expansion
; Create the pixmap
window,/free,/pixmap,xsize = winxsize, ysize = winysize
winpix = !d.window
; Create the visible window
window,0,xsize = winxsize,ysize = winysize
winvis = !d.window

; Set up the display to tile one window with
; two plots: the function+approximation and the
; difference plot.
!p.multi = [0,1,2]
t = fltarr(nterms); integrated difference
yterms = fltarr(nx,nterms); summand
ux = 1+bytarr(nx)
integer = indgen(nterms)
yterms = sin(x#(2.0*integer+1))/(ux#(2.0*integer+1))
for i = 0,nterms-1 do begin
   if i lt 2 then begin
      ysum = yterms[*,0]
   endif else begin
      ysum = total(yterms[*,0:i-1],2)
   endelse
   ; Display the window
   wset,winpix
   plot,x,y,yrange = [-0.5,2.0],ystyle = 1,thick = 2.0, $
      title = strtrim(string(i+1),2)+'!3: Terms'
   oplot,x,0.5*(1.0+(4.0/!pi)*ysum),thick = 1.0
   plot,x,y-0.5*(1.0+(4.0/!pi)*ysum),yrange = [-0.25,0.25],  $
      ystyle = 1,xtitle = 'x',ytitle = 'Difference'
   wset,winvis
   device,copy = [0,0,winxsize,winysize,0,0,winPix]
   empty
   ; Integrate the difference
   t[i] = int_tabulated(x,abs(y-0.5*(1.0+(4.0/!pi)*ysum)))
endfor
wdelete,winpix

!p.multi = 0
window,1,xsize = winxsize,ysize = winysize
plot,t,xtitle = 'x',ytitle = 'Difference'
end