function gaussian,   x,                $
                     area = area,      $
                     center = center,  $
                     fwhm = fwhm,      $
                     plotit = plotit,  $
                     _Extra = extra

; Test for the presence of the input parameter x.
if (n_params() eq 0) or (n_elements(x) eq 0) then $
   x = makepts(xlo = -10.0,xhi = 10.0,npts = 100)
; Test for the presence of the keywords
if n_elements(area) eq 0 then area = 1.0
if n_elements(center) eq 0 then center = 0.0
if n_elements(fwhm) eq 0 then fwhm = 1.0

; Convert the full-width at half-maximum to the
; standard deviation of the Gaussian distribution
sig = fwhm/sqrt(8.0*alog(2.0))
yg = (area/sqrt(2.0*!dpi*sig^2))*exp(-0.5*((x-center)/sig)^2)

; If the keyword PLOTIT has been set then plot the function
; in a new window.
if keyword_set(plotit) then begin
   window,/free,xsize = 300,ysize = 300
   plot,x,yg,psym = -4,xtitle = 'x',ytitle = 'y(x)', $
      title = 'Gaussian Distribution',_Extra = extra
endif
return,yg
end
; ******* TEST_GAUSS *******;
pro test_gauss
device,decomposed = 0
loadct,0,/silent
white = 255 & black = 0
x = makepts(xlo = -5.0,xhi = 5.0,npts = 300)

plot,x,gaussian(x),psym = 0,thick = 2.0,/nodata,color = black, $
   background = white,yrange = [-0.5,1.5],/ysty,   $
   title = '!3Gaussian Distribution', xtitle = 'x',  $
   ytitle = 'y(x)'
oplot,x,gaussian(x),psym = 0,thick = 2.0,color = black

end