;+
; RSI TRAINING FILE HEADER
; FILE: cindex.pro
; ROUTINE: cindex / cindexdraw / cindex_event
; PURPOSE: color table display
; TOPICS: Color lookup tables in IDL
; CLASS: Intro/Intermediate
; REFERENCE: IDL Tour, Color chapter
; LAST MODIFIED: 09/30/99 - BL
; USAGE: .cindex
; NOTES: written by Beau Legeer - RSI training 9/99
;-
PRO cindexdraw, DATA = pState

compile_opt hidden
compile_opt idl2

checkdecompose

; redraw the top image
wset, (*pState).barID
tv, congrid((*pState).barimg, (*pState).xsize, 64)
wset, (*pState).barpixID
tv, congrid((*pState).barimg, (*pState).xsize, 64)

wset, (*pState).drawID

device, get_screen_size = screen

; force the software font
cursysFont = !p.font
!p.font = -1

case screen[0] of
640: fact = 1
800: fact = 1.125
1024: fact = 1.25
1280: fact = 1.625
else: fact = 1.75
endcase

base_s = 320
base_y = (base_x = fix((base_s * fact) / 16))
ysize = (xsize = fix(base_s * fact))

window, /PIXMAP, /FREE, XSIZE = xsize, YSIZE = ysize
tempID = !d.window

base_s = 320
base_y = (base_x = fix((base_s * fact) / 16))
n_color = 0

wset, tempID
for i = 0, 15 do begin
   for j = 0, 15 do begin

      ycoords = [i*base_y, (i+1)*base_y, (i+1)*base_y, i*base_y, i*base_y]
      xcoords = [j*base_x, j*base_x, (j+1)*base_x, (j+1)*base_x, j*base_x]
      polyfill, xcoords, ycoords, COLOR = n_color, /DEVICE

        xyouts, $
           xcoords[0] + 1, $
           ycoords[0] + 3, $
           strtrim(n_color,2), $
           COLOR = abs(n_color - 63), $
           /DEVICE

         n_color = n_color + 1
   endfor
endfor

wset, (*pState).drawID
device, copy = [0,0,xsize, ysize, 0, 0, tempID]

; store back the old font
!p.font = cursysFont

; make a new copy of the pixmap
wset, (*pState).pixID
device, copy = [0,0,xsize, ysize, 0,0,(*pState).drawID]
wset, -1
wdelete, tempID

end

PRO cindex_event, event

compile_opt hidden
compile_opt idl2

WIDGET_CONTROL, event.top, GET_UVALUE = pState
s_name = tag_names(event, /STRUCTURE_NAME)

case s_name of
'WIDGET_BASE': begin

   end

'WIDGET_KILL_REQUEST': begin

   WIDGET_CONTROL, event.top, /DESTROY
   wdelete, (*pState).pixID
   wdelete, (*pState).barpixID
   ptr_free,pState
   return

   end

'WIDGET_KBRD_FOCUS': begin

   ;cindexdraw, DATA = pState

   ; if the widget is gaining focus then set the current
   ; to the widget window
   if (event.enter) then begin
     wset, (*pState).drawID
     cindexdraw, DATA = pState
     wset, (*pState).drawID
     device, copy = [0,0,(*pState).xsize, (*pState).ysize, 0,0, (*pState).pixID]
     wset, -1
   endif else begin
     wset, -1
   endelse

   end

else: begin
   WIDGET_CONTROL, event.id, GET_UVALUE = uval
   case uval of
     'Hide': begin
      WIDGET_CONTROL, (*pState).menuBase, MAP = 0
         WIDGET_CONTROL, (*pState).drawArea, SENSITIVE = 1
         end
     'exit': begin
      WIDGET_CONTROL, event.top, /DESTROY
      wdelete, (*pState).pixID
      wdelete, (*pState).barpixID
      ptr_free,pState
      return
      end
     'ctlist': begin
      if (event.index ne widget_info(event.id, /DROPLIST_NUMBER)-1) then $
         loadct, event.index $
      else $
         tvlct, (*pState).rorig, (*pState).gorig, (*pState).borig
      cindexdraw, DATA = pState
      end
     'redraw': begin
      cindexdraw, DATA = pState
      end
     'xloadct': begin
      wset, (*pState).drawID
      xloadct, group = event.top, UPDATECALLBACK = 'cindexdraw', $
         UPDATECBDATA = pState, /MODAL
         end
;   'xpalette': begin
;      wset, (*pState).drawID
;      xpalette2, group = event.top, UPDATECALLBACK = 'cindexdraw', $
;        /BLOCK,  UPDATECBDATA = pState, /MODAL
;       end
      'sysinfo': begin
         syscolorinfo, GROUP_LEADER = event.top
         end
   endcase
   end
endcase

return

end

PRO cindex_maindraw, event

widget_control, event.top, get_uvalue = pState

case event.type of
2: begin

    j = event.x / (*pState).base_x
    i = event.y / (*pState).base_y

    n_color = (i * 16) + j
    if (n_color gt !d.table_size-1) then n_color = !d.table_size-1

    if (n_color ne (*pState).prevcolor) then begin
        cindex_updatestatus, pState, n_color

        (*pState).prevcolor = n_color
        wset, (*pState).drawID

        checkdecompose

        device, copy = [0,0,(*pState).xsize, (*pState).ysize, 0, 0, (*pState).pixID]

     wset, (*pState).barID
     device, copy = [0,0,(*pState).xsize, 64, 0, 0, (*pState).barpixID]
     xbc = n_color * ((*pState).xsize / 256.)
     plots, [xbc, xbc], [0, 63], /device, color = !d.table_size-1, THICK=3
     wset, (*pState).drawID

        ; create new coords and draw a bigger box
        base_x = (*pState).base_x
        base_y = (*pState).base_y

        ycoords = [i*base_y - 10, $
           (i+1)*base_y + 10, $
           (i+1)*base_y + 10, $
           i*base_y - 10, $
           i*base_y - 10]
        xcoords = [j*base_x - 10, $
           j*base_x - 10, $
           (j+1)*base_x + 10, $
           (j+1)*base_x + 10, $
           j*base_x - 10]
        polyfill, xcoords, ycoords, COLOR = n_color, /DEVICE
        plots, xcoords, ycoords, COLOR = 255, /DEVICE
        xyouts, $
           xcoords[0] + 1, $
           ycoords[0] + 13, $
           '!17'+strtrim(n_color,2)+'!X', $
           COLOR = abs(n_color - 191), $
           CHARSIZE = 2, $
           /DEVICE
   endif
   wset, -1
   end

0: begin
   (*pState).mode = event.press
   end

1: begin
   end

3:
4:
endcase

END

PRO cindex_topdraw, event

if (event.type eq 0) then (*pState).mode = event.press

widget_control, event.top, get_uvalue = pState
n_color = fix(round(255 * (float(event.x)/(*pState).xsize)))
if (n_color gt !d.table_size-1) then n_color = !d.table_size-1

j = n_color mod 16
i = n_color / 16

cindex_updatestatus, pState, n_color

wset, (*pState).drawID

checkdecompose
device, copy = [0,0,(*pState).xsize, (*pState).ysize, 0, 0, (*pState).pixID]

wset, (*pState).barID
device, copy = [0,0,(*pState).xsize, 64, 0, 0, (*pState).barpixID]
plots, [event.x, event.x], [0, 63], /device, color = byte(n_color) - 160

wset, (*pState).drawID

; create new coords and draw a bigger box
base_x = (*pState).base_x
base_y = (*pState).base_y

ycoords = [i*base_y - 10, $
   (i+1)*base_y + 10, $
   (i+1)*base_y + 10, $
   i*base_y - 10, $
    i*base_y - 10]
xcoords = [j*base_x - 10, $
    j*base_x - 10, $
    (j+1)*base_x + 10, $
    (j+1)*base_x + 10, $
    j*base_x - 10]
polyfill, xcoords, ycoords, COLOR = n_color, /DEVICE
plots, xcoords, ycoords, COLOR = 255, /DEVICE
xyouts, $
   xcoords[0] + 1, $
   ycoords[0] + 13, $
    '!17'+strtrim(n_color,2)+'!X', $
    COLOR = abs(n_color - 191), $
    CHARSIZE = 2, $
    /DEVICE
wset, -1

END

PRO cindex_updatestatus, pstate, n_color

tvlct, r, g, b, /get
case (*pState).mode of
1: begin
WIDGET_CONTROL, $
    (*pState).statusLabel, $
    SET_VALUE = 'COLOR: '+strtrim(n_color,2)+'   '+ $
    ' RED: '+strtrim(fix(r[n_color]), 2)+ $
    '   GREEN: '+strtrim(fix(g[n_color]), 2)+ $
    '   BLUE: '+strtrim(fix(b[n_color]), 2)
end

2: begin
color_convert, r[n_color], g[n_color], b[n_color], $
    h,l,s, /RGB_HLS
WIDGET_CONTROL, (*pState).statusLabel, $
    SET_VALUE = 'COLOR: '+strtrim(n_color,2)+'   '+ $
    ' HUE: '+strtrim(fix(h), 2)+ $
    '   LIGHTNESS: '+strtrim(fix(l), 2)+ $
    '   SATURATION: '+strtrim(fix(s), 2)
end

4: begin
color_convert, r[n_color], g[n_color], b[n_color],$
    h,s,v, /RGB_HSV
WIDGET_CONTROL, $
    (*pState).statusLabel, $
    SET_VALUE = 'COLOR: '+strtrim(n_color,2)+'   '+ $
    ' HUE: '+strtrim(fix(h), 2)+ $
    '   SATURATION: '+strtrim(fix(s), 2)+ $
    '   VALUE: '+strtrim(fix(v), 2)
end
else:
endcase

end

;**********************************************************
;pro cindexCleanup,tlb
;widget_control,tlb,get_uvalue = pState
;ptr_free,pState
;end
;**********************************************************

PRO cindex, $
    xsize = xsize, $
    ysize = ysize, $
    nosysinfo = nosysinfo, $
    allowmultiple = allowmultiple

compile_opt hidden
compile_opt idl2

if (not keyword_set(allowmultiple)) then begin
   if (xregistered('cindex')) then return
endif

checkdecompose, OLD_DC = oldDC

; get the current screen size
device, get_screen_size = screen

; figure out factors for drawing the color
; table in the window
case screen[0] of
640: fact = 1
800: fact = 1.125
1024: fact = 1.25
1280: fact = 1.625
else: fact = 1.75
endcase

base_s = 320
base_y = (base_x = fix((base_s * fact) / 16))
ysize = (xsize = fix(base_s * fact))

; get the current color table
tvlct, rorig, gorig, borig, /GET

; create the widgets
tlb = WIDGET_BASE(TITLE = 'Color Index Numbers', $
   /TLB_KILL_REQUEST_EVENTS, $
   /KBRD_FOCUs_EVENTS, $
   TLB_FRAME_ATTR = 1, $
   /COLUMN, MBAR = menubase)

fileMenu = widget_button(menubase, VALUE = 'File', $
   /MENU)
if (not keyword_set(nosysinfo)) then begin
   sysinfoButton = widgeT_button(fileMenu, VALUE = 'System Color Information', $
     UVALUE = 'sysinfo')
endif
closeButton = WIDGET_BUTTON(fileMenu, $
   VALUE =  ' Exit ', $
   UVALUE = 'exit')


controlMenu = WIDGET_BUTTON(menuBase, $
   VALUE = 'Color Table', $
   /MENU)
redrawButton = WIDGET_BUTTON(ControlMenu, $
   VALUE = ' Redraw ', $
   UVALUE = 'redraw')
xloadctButton = WIDGET_BUTTON(ControlMenu, $
   VALUE = ' XloadCt ', $
   UVALUE = 'xloadct')
;xpaletteButton = WIDGET_BUTTON(ControlMenu, $
;  VALUE = ' XPalette ', $
;  UVALUE = 'xpalette')

mainBB = WIDGET_BASE(tlb, /COLUMN)

draw1 = widget_draw(mainBB, XSIZE = xsize, $
   YSIZE = 64, /MOTION_EVENTS, $
   EVENT_PRO = 'cindex_topdraw')

row1 = widget_base(mainBB, /ROW, /ALIGN_CENTER)
loadct, get_names = names
names = [names, 'Entry Color Table']
for i = 0, n_elements(names)-1 do names[i] = strtrim(i,2)+'. '+names[i]
ctlist = widget_droplist(row1, VALUE = names, $
   TITLE = 'Load Predefined Color Table: ', $
   UVALUE = 'ctlist')
widget_control, ctlist, SET_DROPLIST_SELECT = n_elements(names)-1
; draw area widget
drawArea = WIDGET_DRAW(mainBB, $
   XSIZE = xsize, $
   YSIZE = ysize, $
   /MOTION_EVENTS, $
   /BUTTON_EVENTS, $
   EVENT_PRO = 'cindex_maindraw')

statusRow = widget_base(tlb, /row)
statusLabel = WIDGET_LABEL(statusrow, $
   VALUE = 'Move mouse over colors to begin', $
   /DYNAMIC_RESIZE)

; save the value of the windowID before the creating the draw area
oldID = !d.window

WIDGET_CONTROL, tlb, /REALIZE
WIDGET_CONTROL, tlb, /MANAGED
WIDGET_CONTROL, drawArea, GET_VALUE = drawID
WIDGET_CONTROL, draw1, GET_VALUE = barID

wset, barID
barimg = bytarr(256,64)
for i = 0, 255 do barimg[i,*]=i
tv, congrid(barimg, xsize, 64)

window, /FREE, /PIXMAP, XSIZE = xsize, YSIZE = 64
barpixID = !d.window
tv, congrid(barimg, xsize, 64)

wset, drawID

state = {xsize:xsize, $
   ysize:ysize, $
   barimg:barimg, $
   base_x:base_x, $
   base_y:base_y, $
   drawArea:drawArea, $
   barID:barID, $
   oldID:oldID, $
   pixID:0, $
   barpixID:barpixID, $
   drawID:drawID, $
   prevcolor:0, $
   statusLabel:statusLabel, $
   rorig:rorig, $
   gorig:gorig, $
   borig:borig, $
   mode:1}
pState = ptr_new(state, /no_copy)

; create an offscreen pixmap and copy the colormap
window, $
   /FREE, $
   /PIXMAP, $
   XSIZE = xsize, $
   YSIZE = ysize
(*pState).pixID = !d.window

cindexdraw, DATA = pState
WIDGET_CONTROL, tlb, SET_UVALUE = pState

Xmanager, 'cindex', tlb, /NO_BLOCK

END

